Package: oskeyring
Title: Raw System Credential Store Access from R
Version: 0.1.7
Authors@R: c(
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = c("aut", "cre")),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Aims to support all features of the system credential store,
    including non-portable ones. Supports 'Keychain' on 'macOS', and
    'Credential Manager' on 'Windows'. See the 'keyring' package if you
    need a portable 'API'.
License: MIT + file LICENSE
URL: https://github.com/r-lib/oskeyring#readme,
        https://r-lib.github.io/oskeyring/
BugReports: https://github.com/r-lib/oskeyring/issues
Depends: R (>= 3.6)
Suggests: askpass, covr, testthat (>= 3.0.0), withr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/usethis/last-upkeep: 2025-04-30
Encoding: UTF-8
RoxygenNote: 7.2.3
NeedsCompilation: yes
Packaged: 2026-04-08 08:35:18 UTC; gaborcsardi
Author: Gábor Csárdi [aut, cre],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-08 09:00:09 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 20:59:23 UTC; unix
Archs: oskeyring.so.dSYM
