# Generated by roxygen2: do not edit by hand

S3method(format,oskeyring_macos_item)
S3method(format,oskeyring_windows_item)
S3method(print,oskeyring_macos_item)
S3method(print,oskeyring_windows_item)
export(macos_item)
export(macos_item_add)
export(macos_item_attr)
export(macos_item_classes)
export(macos_item_delete)
export(macos_item_match_options)
export(macos_item_search)
export(macos_item_update)
export(macos_keychain_create)
export(macos_keychain_delete)
export(macos_keychain_is_locked)
export(macos_keychain_list)
export(macos_keychain_lock)
export(macos_keychain_unlock)
export(windows_item)
export(windows_item_delete)
export(windows_item_enumerate)
export(windows_item_read)
export(windows_item_types)
export(windows_item_write)
useDynLib(oskeyring, .registration = TRUE)
